@@ -10,8 +10,8 @@
 #           - With or without a "Approach Release" Button
 #           - Main signal manual or fully automatic
 #    Semaphore Signals - Home or Distant
-#           - with or without junction arms (RH and LH arms supported)
-#           - with or without subsidaries (Main, LH or RH arms supported - for Home signals only)
+#           - with or without junction arms (RH1, RH2, LH1, LH2 arms supported)
+#           - with or without subsidaries (Main, LH1, LH2, RH1, RH2 arms supported) - Home signals only
 #           - with or without a theatre type route indicator (for Home signals only)
 #           - With or without a "Signal Passed" Button
 #           - With or without a "Approach Release" Button
@@ -39,9 +39,6 @@
 #   route_type.RH1    (immediate right)
 #   route_type.RH2    (rar right)
 # These equate to the route feathers for colour light signals or the Sempahore junction "arm":
-#   RH1 or RH2 make the RH junction arm and RH subsidary arm "active"
-#   LH1 or LH2 make the LH junction arm and LH subsidary arm "active"
-#   MAIN or NONE make the "main" signal arm and the "main" subsidary arm "active"
 # 
 # sig_callback_type (tells the calling program what has triggered the callback):
 #     sig_callback_type.sig_switched (signal has been switched)
@@ -54,7 +51,7 @@
 #   Mandatory Parameters:
 #       Canvas - The Tkinter Drawing canvas on which the point is to be displayed
 #       sig_id:int - The ID for the signal - also displayed on the signal button
-#       x:int, y:int - Position of the point on the canvas (in pixels) 
+#       x:int, y:int - Position of the signal on the canvas (in pixels) 
 #   Optional Parameters:
 #       signal_subtype:sig_sub_type - type of signal to create - Default is signal_sub_type.four_aspect
 #       orientation:int- Orientation in degrees (0 or 180) - Default is zero
@@ -79,41 +76,42 @@
 #   Mandatory Parameters:
 #       Canvas - The Tkinter Drawing canvas on which the point is to be displayed
 #       sig_id:int - The ID for the signal - also displayed on the signal button
-#       x:int, y:int - Position of the point on the canvas (in pixels) 
+#       x:int, y:int - Position of the signal on the canvas (in pixels) 
 #   Optional Parameters:
-#       distant:bool - Set to True to create a Distant signal - False to create a Home signal - default False
+#       distant:bool - True to create a Distant signal - False to create a Home signal - default False
+#       associated_home:bool - Option only valid when creating distant signals - Set to True to associate
+#                              the distant signal with a previously created home signal - default False
+#                              (this option enables a distant signal to "share" the same post as the
+#                               home signal - specify the same x and y coordinates as the home signal) 
 #       orientation:int - Orientation in degrees (0 or 180) - Default is zero
 #       sig_callback:name - Function to call when a signal event happens - Default is no callback
 #                           Note that the callback function returns (item_id, callback type)
 #       sig_passed_button:bool - Creates a "signal Passed" button for automatic control - Default False
-#       approach_release_button:bool - Creates an "Approach Release" button - Default False
-#       lh1_signal:bool - To create a LH1 post with a main (junction) signal - default False
-#       lh2_signal:bool - To create a LH2 post with a main (junction) signal - default False
-#       rh1_signal:bool - To create a RH1 post with a main (junction) signal - default False
-#       rh2_signal:bool - To create a RH2 post with a main (junction) signal - default False
+#       approach_release_button:bool - Option only valid for home signals - Creates an "Approach Release"
+#                                      button in front of the signal - Default False
+#       main_signal:bool - To create a signal arm for the main route - default True
+#                       (Only set this to False for the case of creating a distant signal "associated 
+#                        with" a home signal where a distant arm for the main route is not required)
+#       lh1_signal:bool - To create a LH1 post with a main (junction) arm - default False
+#       lh2_signal:bool - To create a LH2 post with a main (junction) arm - default False
+#       rh1_signal:bool - To create a RH1 post with a main (junction) arm - default False
+#       rh2_signal:bool - To create a RH2 post with a main (junction) arm - default False
 #       main_subsidary:bool - To create a subsidary signal under the "main" signal - default False
-#       lh1_subsidary:bool - To create a LH1 post with a subsidary signal - default False
-#       lh2_subsidary:bool - To create a LH2 post with a subsidary signal - default False
-#       rh1_subsidary:bool - To create a RH1 post with a subsidary signal - default False
-#       rh2_subsidary:bool - To create a RH2 post with a subsidary signal - default False
-#       main_distant:bool - To create a secondary distant signal (for the signal ahead) - default False
-#       lh1_distant:bool - To create a LH1 secondary distant signal (for the signal ahead) - default False
-#       lh2_distant:bool - To create a LH2 secondary distant signal (for the signal ahead) - default False
-#       rh1_distant:bool - To create a RH1 secondary distant signal (for the signal ahead) - default False
-#       rh2_distant:bool - To create a RH2 secondary distant signal (for the signal ahead) - default False
+#       lh1_subsidary:bool - To create a LH1 post with a subsidary arm - default False
+#       lh2_subsidary:bool - To create a LH2 post with a subsidary arm - default False
+#       rh1_subsidary:bool - To create a RH1 post with a subsidary arm - default False
+#       rh2_subsidary:bool - To create a RH2 post with a subsidary arm - default False
 #       theatre_route_indicator:bool -  Creates a Theatre Type route indicator - Default False
-#       refresh_immediately:bool - When set to False the signal aspects will NOT be automaticall updated 
+#       refresh_immediately:bool - When set to False the signal aspects will NOT be automatically updated 
 #                 when the signal is changed and the external programme will need to call the seperate 
-#                 'update_signal' function. Primarily intended for use with home signals that have a
-#                 secondary distant arm, which will reflect the state of the signal ahead (i.e. if the
-#                 signal ahead is at DANGER then the secondary distant arm will be ON) - Default True
+#                 'update_signal' function. Primarily intended for use with distant signals - Default True
 #       fully_automatic:bool - Creates a signal without any manual controls - Default False
 # 
 # create_ground_position_signal - create a ground position light signal
 #   Mandatory Parameters:
 #       Canvas - The Tkinter Drawing canvas on which the point is to be displayed
 #       sig_id:int - The ID for the signal - also displayed on the signal button
-#       x:int, y:int - Position of the point on the canvas (in pixels) 
+#       x:int, y:int - Position of the signal on the canvas (in pixels) 
 #   Optional Parameters:
 #       orientation:int- Orientation in degrees (0 or 180) - Default is zero
 #       sig_callback:name - Function to call when a signal event happens - Default is no callback
@@ -126,7 +124,7 @@
 #   Mandatory Parameters:
 #       Canvas - The Tkinter Drawing canvas on which the point is to be displayed
 #       sig_id:int - The ID for the signal - also displayed on the signal button
-#       x:int, y:int - Position of the point on the canvas (in pixels) 
+#       x:int, y:int - Position of the signal on the canvas (in pixels) 
 #  Optional Parameters:
 #       orientation:int- Orientation in degrees (0 or 180) - Default is zero
 #       sig_callback:name - Function to call when a signal event happens - Default is no callback
@@ -213,56 +211,6 @@ from . import signals_semaphores
 from tkinter import *
 import logging
 
-# -------------------------------------------------------------------------
-# Externally called Function to update a signal according the state of the
-# Signal ahead - Intended mainly for Coulour Light Signal types so we can
-# ensure the "CLEAR" aspect reflects the aspect of ths signal ahead
-# Calls the signal type-specific functions depending on the signal type
-# Function applicable only to main Colour Light signal types
-# -------------------------------------------------------------------------
-
-def update_signal (sig_id:int, sig_ahead_id:int = 0):
-    
-    global logging
-    # Validate the signal exists (and the one ahead if specified)
-    if not signals_common.sig_exists(sig_id):
-        logging.error ("Signal "+str(sig_id)+": update_signal - Signal does not exist")
-    elif sig_ahead_id != 0 and not signals_common.sig_exists(sig_ahead_id): 
-        logging.error ("Signal "+str(sig_id)+": update_signal - Signal ahead "+str(sig_ahead_id)+" does not exist")
-    elif sig_id == sig_ahead_id: 
-        logging.error ("Signal "+str(sig_id)+": update_signal - Signal ahead "+str(sig_ahead_id)+" is the same ID")
-    # Call the signal type-specific functions to update the signal (only colour lights supported currently)
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
-        signals_colour_lights.update_colour_light_signal_aspect (sig_id,sig_ahead_id )
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
-        signals_semaphores.update_semaphore_signal (sig_id,sig_ahead_id )
-    else:
-        logging.error ("Signal "+str(sig_id)+": update_signal - Function not supported by signal type")
-    return()
-
-# -------------------------------------------------------------------------
-# Externally called function to set the route indication for the signal
-# Calls the signal type-specific functions depending on the signal type
-# Function applicable to Main Colour Light and Semaphore signal types
-# -------------------------------------------------------------------------
-
-def set_route (sig_id:int, route:signals_common.route_type = None, theatre_text:str = None):
-    
-    global logging
-    # Validate the signal exists
-    if not signals_common.sig_exists(sig_id):
-        logging.error ("Signal "+str(sig_id)+": set_route - Signal does not exist")
-        # Call the signal type-specific functions to update the signal
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
-        signals_colour_lights.update_feather_route_indication (sig_id,route)
-        signals_common.update_theatre_route_indication(sig_id,theatre_text)
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
-        signals_semaphores.update_semaphore_route_indication (sig_id,route)
-        signals_common.update_theatre_route_indication(sig_id,theatre_text)
-    else:
-        logging.error ("Signal "+str(sig_id)+": set_route - Function not supported by signal type")
-    return()
-
 # -------------------------------------------------------------------------
 # Externally called function to Return the current state of the signal
 # Function applicable to ALL signal types
@@ -400,7 +348,7 @@ def lock_subsidary (*sig_ids:int):
         elif not signals_common.signals[str(sig_id)]["hassubsidary"]:
             logging.error ("Signal "+str(sig_id)+": lock_subsidary - Signal does not have a subsidary")
         # Only lock if the signal has a subsidary and it is currently unlocked
-        elif signals_common.signals[str(sig_id)]["hassubsidary"] and not signals_common.signals[str(sig_id)]["sublocked"]:
+        elif not signals_common.signals[str(sig_id)]["sublocked"]:
             logging.info ("Signal "+str(sig_id)+": Locking subsidary")
             # If signal/point locking has been correctly implemented it should
             # only be possible to lock a signal that is "ON" (i.e. at DANGER)
@@ -428,7 +376,7 @@ def unlock_subsidary (*sig_ids:int):
         elif not signals_common.signals[str(sig_id)]["hassubsidary"]:
             logging.error ("Signal "+str(sig_id)+": unlock_subsidary - Signal does not have a subsidary")
         # Only unlock if the signal has a subsidary and it is currently locked
-        elif signals_common.signals[str(sig_id)]["hassubsidary"] and signals_common.signals[str(sig_id)]["sublocked"]:
+        elif signals_common.signals[str(sig_id)]["sublocked"]:
             logging.info ("Signal "+str(sig_id)+": Unlocking subsidary")
             # Re-enable the Button to unlock the subsidary signal
             signals_common.signals[str(sig_id)]["subbutton"].config(state="normal")
@@ -463,7 +411,7 @@ def set_signal_override (*sig_ids:int):
                 # Otherwise, it will be the responsibility of the calling programme to make another
                 # call to update the signal aspect accordingly (based on the signal ahead)
                 if signals_common.signals[str(sig_id)]["refresh"]:
-                    signals_colour_lights.update_colour_light_signal_aspect(sig_id)
+                    signals_colour_lights.update_colour_light_signal(sig_id)
             elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
                 # We only refresh Semaphore signals if they are configured to refresh when switched
                 # Otherwise, it will be the responsibility of the calling programme to make another
@@ -471,10 +419,9 @@ def set_signal_override (*sig_ids:int):
                 if signals_common.signals[str(sig_id)]["refresh"]:
                     signals_semaphores.update_semaphore_signal (sig_id)
             elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.ground_position:
-                signals_ground_position.update_ground_position_light_signal (sig_id)
+                signals_ground_position.update_ground_position_signal (sig_id)
             elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.ground_disc:
-                signals_ground_disc.update_ground_disc_signal (sig_id)
-                    
+                signals_ground_disc.update_ground_disc_signal (sig_id)  
         return()
 
 # -------------------------------------------------------------------------
@@ -497,14 +444,13 @@ def clear_signal_override (*sig_ids:int):
             # Clear the override and change the button colour
             signals_common.signals[str(sig_id)]["override"] = False
             signals_common.signals[str(sig_id)]["sigbutton"].config(fg="black",disabledforeground="grey50")
-            
             # now call the signal type-specific functions to update the signal
             if signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
                 # We only refresh colour light signals if they are configured to refresh when switched
                 # Otherwise, it will be the responsibility of the calling programme to make another
                 # call to update the signal aspect accordingly (based on the signal ahead)
                 if signals_common.signals[str(sig_id)]["refresh"]:
-                    signals_colour_lights.update_colour_light_signal_aspect(sig_id)
+                    signals_colour_lights.update_colour_light_signal(sig_id)
             elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
                 # We only refresh Semaphore signals if they are configured to refresh when switched
                 # Otherwise, it will be the responsibility of the calling programme to make another
@@ -512,12 +458,161 @@ def clear_signal_override (*sig_ids:int):
                 if signals_common.signals[str(sig_id)]["refresh"]:
                     signals_semaphores.update_semaphore_signal (sig_id)
             elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.ground_position:
-                signals_ground_position.update_ground_position_light_signal (sig_id)
+                signals_ground_position.update_ground_position_signal (sig_id)
             elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.ground_disc:
                 signals_ground_disc.update_ground_disc_signal (sig_id)
-                
     return() 
 
+# -------------------------------------------------------------------------
+# Externally called function to Toggle the state of a main signal
+# to enable automated route setting from the external programme.
+# Use in conjunction with 'signal_clear' to find the state first
+# Function applicable to ALL Signal Types
+# -------------------------------------------------------------------------
+
+def toggle_signal (sig_id:int):
+    
+    global logging
+    # Validate the signal exists
+    if not signals_common.sig_exists(sig_id):
+        logging.error ("Signal "+str(sig_id)+": toggle_signal - Signal does not exist")
+    elif signals_common.signals[str(sig_id)]["automatic"]:
+        logging.error ("Signal "+str(sig_id)+": toggle_signal - Signal is automatic - use set/clear signal override instead")
+    else:
+        if signals_common.signals[str(sig_id)]["siglocked"]:
+            logging.warning ("Signal "+str(sig_id)+": toggle_signal - Signal is locked - Toggling anyway")
+        # call the signal type-specific functions to update the signal
+        signals_common.toggle_signal(sig_id)
+    return()
+
+# -------------------------------------------------------------------------
+# Externally called function to Toggle the state of a subsidary signal
+# to enable automated route setting from the external programme. Use
+# in conjunction with 'subsidary_signal_clear' to find the state first
+# Function applicable to ALL Signal Types (if they "hasvesubsidary")
+# -------------------------------------------------------------------------
+
+def toggle_subsidary (sig_id:int):
+    
+    global logging
+    # Validate the signal exists
+    if not signals_common.sig_exists(sig_id):
+        logging.error ("Signal "+str(sig_id)+": toggle_subsidary - Signal does not exist")
+    elif not signals_common.signals[str(sig_id)]["hassubsidary"]:
+        logging.error ("Signal "+str(sig_id)+": toggle_subsidary - Signal does not have a subsidary")
+    else:
+        if signals_common.signals[str(sig_id)]["sublocked"]:
+            logging.warning ("Signal "+str(sig_id)+": toggle_subsidary - Subsidary signal is locked - Toggling anyway")
+        #  call the signal type-specific functions to update the signal
+        signals_common.toggle_subsidary(sig_id)
+    return()
+
+# -------------------------------------------------------------------------
+# Externally called function to set the "approach conrol" for the signal
+# Calls the signal type-specific functions depending on the signal type
+# Function applicable to Colour Light and Semaphore signal types
+# -------------------------------------------------------------------------
+
+def set_approach_control (sig_id:int, release_on_yellow:bool = False):
+    
+    global logging
+    # Validate the signal exists
+    if not signals_common.sig_exists(sig_id):
+        logging.error ("Signal "+str(sig_id)+": set_approach_control - Signal does not exist")
+    # call the signal type-specific functions to update the signal
+    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
+        # do some additional validation specific to this function for colour light signals
+        if signals_common.signals[str(sig_id)]["subtype"]==signals_colour_lights.signal_sub_type.distant:
+            logging.error("Signal "+str(sig_id)+": Can't set approach control for a 2 aspect distant signal")
+        elif release_on_yellow and signals_common.signals[str(sig_id)]["subtype"]==signals_colour_lights.signal_sub_type.home:
+            logging.error("Signal "+str(sig_id)+": Can't set \'release on yellow\' approach control for a 2 aspect home signal")
+        elif release_on_yellow and signals_common.signals[str(sig_id)]["subtype"]==signals_colour_lights.signal_sub_type.red_ylw:
+            logging.error("Signal "+str(sig_id)+": Can't set \'release on yellow\' approach control for a 2 aspect red/yellow signal")
+        else:
+            signals_common.set_approach_control(sig_id,release_on_yellow)            
+    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
+        # Do some additional validation specific to this function for semaphore signals
+        if signals_common.signals[str(sig_id)]["distant"]:
+            logging.error("Signal "+str(sig_id)+": Can't set approach control for semaphore distant signals")
+        elif release_on_yellow:
+            logging.error("Signal "+str(sig_id)+": Can't set \'release on yellow\' approach control for home signals")
+        else:
+            signals_common.set_approach_control(sig_id)
+    else:
+        logging.error ("Signal "+str(sig_id)+": set_approach_control - Function not supported by signal type")
+    return()
+
+# -------------------------------------------------------------------------
+# Externally called function to clear the "approach control" for the signal
+# Calls the signal type-specific functions depending on the signal type
+# Function applicable to Colour Light and Semaphore signal types
+# -------------------------------------------------------------------------
+
+def clear_approach_control (sig_id:int):
+    
+    global logging
+    # Validate the signal exists
+    if not signals_common.sig_exists(sig_id):
+        logging.error ("Signal "+str(sig_id)+": clear_approach_control - Signal does not exist")  
+    # Call the signal type-specific functions to update the signal
+    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
+        signals_common.clear_approach_control (sig_id)
+    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
+        signals_common.clear_approach_control (sig_id)
+    else:
+        logging.error ("Signal "+str(sig_id)+": clear_approach_control - Function not supported by signal type")
+    return()
+
+# -------------------------------------------------------------------------
+# Externally called Function to update a signal according the state of the
+# Signal ahead - Intended mainly for Coulour Light Signal types so we can
+# ensure the "CLEAR" aspect reflects the aspect of ths signal ahead
+# Calls the signal type-specific functions depending on the signal type
+# Function applicable only to main Colour Light signal types
+# -------------------------------------------------------------------------
+
+def update_signal (sig_id:int, sig_ahead_id:int = 0):
+    
+    global logging
+    # Validate the signal exists (and the one ahead if specified)
+    if not signals_common.sig_exists(sig_id):
+        logging.error ("Signal "+str(sig_id)+": update_signal - Signal does not exist")
+    elif sig_ahead_id != 0 and not signals_common.sig_exists(sig_ahead_id): 
+        logging.error ("Signal "+str(sig_id)+": update_signal - Signal ahead "+str(sig_ahead_id)+" does not exist")
+    elif sig_id == sig_ahead_id: 
+        logging.error ("Signal "+str(sig_id)+": update_signal - Signal ahead "+str(sig_ahead_id)+" is the same ID")
+    # Call the signal type-specific functions to update the signal (only colour lights supported currently)
+    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
+        signals_colour_lights.update_colour_light_signal (sig_id,sig_ahead_id )
+    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
+        signals_semaphores.update_semaphore_signal (sig_id,sig_ahead_id )
+    else:
+        logging.error ("Signal "+str(sig_id)+": update_signal - Function not supported by signal type")
+    return()
+
+# -------------------------------------------------------------------------
+# Externally called function to set the route indication for the signal
+# Calls the signal type-specific functions depending on the signal type
+# Function applicable to Main Colour Light and Semaphore signal types
+# -------------------------------------------------------------------------
+
+def set_route (sig_id:int, route:signals_common.route_type = None, theatre_text:str = None):
+    
+    global logging
+    # Validate the signal exists
+    if not signals_common.sig_exists(sig_id):
+        logging.error ("Signal "+str(sig_id)+": set_route - Signal does not exist")
+        # Call the signal type-specific functions to update the signal
+    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
+        signals_colour_lights.update_feather_route_indication (sig_id,route)
+        signals_common.update_theatre_route_indication(sig_id,theatre_text)
+    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
+        signals_semaphores.update_semaphore_route_indication (sig_id,route)
+        signals_common.update_theatre_route_indication(sig_id,theatre_text)
+    else:
+        logging.error ("Signal "+str(sig_id)+": set_route - Function not supported by signal type")
+    return()
+
 # -------------------------------------------------------------------------
 # Externally called Function to 'override' a signal (changing it to 'ON') after
 # a specified time delay and then clearing the override the signal after another
@@ -547,140 +642,7 @@ def trigger_timed_signal (sig_id:int,start_delay:int=0,time_delay:int=5):
         logging.info ("Signal "+str(sig_id)+": Triggering Timed Signal")
         signals_semaphores.trigger_timed_semaphore_signal (sig_id,start_delay,time_delay)
     else:
-        logging.error ("Signal "+str(sig_id)+": set_route - Function not supported by signal type")
-    return()
-
-# -------------------------------------------------------------------------
-# Externally called function to Toggle the state of a main signal
-# to enable automated route setting from the external programme.
-# Use in conjunction with 'signal_clear' to find the state first
-# Function applicable to ALL Signal Types
-# -------------------------------------------------------------------------
-
-def toggle_signal (sig_id:int):
-    
-    global logging
-    # Validate the signal exists
-    if not signals_common.sig_exists(sig_id):
-        logging.error ("Signal "+str(sig_id)+": toggle_signal - Signal does not exist")
-    else:
-        if signals_common.signals[str(sig_id)]["siglocked"]:
-            logging.warning ("Signal "+str(sig_id)+": toggle_signal - Signal is locked - Toggling anyway")
-        # call the signal type-specific functions to update the signal
-        if signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
-            signals_colour_lights.toggle_colour_light_signal(sig_id)
-        elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.ground_position:
-            signals_ground_position.toggle_ground_position_light_signal (sig_id)
-        elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
-            signals_semaphores.toggle_semaphore_signal(sig_id)
-        elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.ground_disc:
-            signals_ground_disc.toggle_ground_disc_signal (sig_id)
-    return()
-
-# -------------------------------------------------------------------------
-# Externally called function to Toggle the state of a subsidary signal
-# to enable automated route setting from the external programme. Use
-# in conjunction with 'subsidary_signal_clear' to find the state first
-# Function applicable to ALL Signal Types (if they "hasvesubsidary")
-# -------------------------------------------------------------------------
-
-def toggle_subsidary (sig_id:int):
-    
-    global logging
-    # Validate the signal exists
-    if not signals_common.sig_exists(sig_id):
-        logging.error ("Signal "+str(sig_id)+": toggle_subsidary - Signal does not exist")
-    elif not signals_common.signals[str(sig_id)]["hassubsidary"]:
-        logging.error ("Signal "+str(sig_id)+": toggle_subsidary - Signal does not have a subsidary")
-    else:
-        if signals_common.signals[str(sig_id)]["sublocked"]:
-            logging.warning ("Signal "+str(sig_id)+": toggle_subsidary - Subsidary signal is locked - Toggling anyway")
-        #  call the signal type-specific functions to update the signal
-        if signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
-            signals_colour_lights.toggle_colour_light_subsidary(sig_id)
-        elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
-            signals_semaphores.toggle_semaphore_subsidary(sig_id)
-    return()
-
-# -------------------------------------------------------------------------
-# Externally called function to set the "approach conrol" for the signal
-# Calls the signal type-specific functions depending on the signal type
-# Function applicable to Colour Light and Semaphore signal types
-# -------------------------------------------------------------------------
-
-def set_approach_control (sig_id:int, release_on_yellow:bool = False):
-    
-    global logging
-    # Validate the signal exists
-    if not signals_common.sig_exists(sig_id):
-        logging.error ("Signal "+str(sig_id)+": set_approach_control - Signal does not exist")
-    # call the signal type-specific functions to update the signal
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
-        signals_colour_lights.set_approach_control(sig_id,release_on_yellow)
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
-        signals_semaphores.set_approach_control(sig_id,release_on_yellow)
-    else:
-        logging.error ("Signal "+str(sig_id)+": set_approach_control - Function not supported by signal type")
-    return()
-
-# -------------------------------------------------------------------------
-# Externally called function to clear the "approach control" for the signal
-# Calls the signal type-specific functions depending on the signal type
-# Function applicable to Colour Light and Semaphore signal types
-# -------------------------------------------------------------------------
-
-def clear_approach_control (sig_id:int):
-    
-    global logging
-    # Validate the signal exists
-    if not signals_common.sig_exists(sig_id):
-        logging.error ("Signal "+str(sig_id)+": clear_approach_control - Signal does not exist")  
-    # Call the signal type-specific functions to update the signal
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
-        signals_colour_lights.clear_approach_control (sig_id)
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
-        signals_semaphores.clear_approach_control (sig_id)
-    else:
-        logging.error ("Signal "+str(sig_id)+": clear_approach_control - Function not supported by signal type")
-    return()
-
-# -------------------------------------------------------------------------
-# Functions called from the Sensors Module to trigger either signal passed
-# otr signal released events automatically when a sensor is triggered
-# Calls the signal type-specific functions depending on the signal type
-# Note that these are internal functions - not part of the public API
-# -------------------------------------------------------------------------
-
-def trigger_signal_passed_event (sig_id:int):
-
-    global logging
-    # Validate the signal exists
-    if not signals_common.sig_exists(sig_id):
-        logging.error ("Signal "+str(sig_id)+": trigger_signal_passed_event - Signal does not exist")
-    # call the signal type-specific functions to update the signal
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
-        signals_colour_lights.sig_passed_button_event(sig_id)
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.ground_position:
-        signals_ground_position.sig_passed_button_event(sig_id)
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
-        signals_semaphores.sig_passed_button_event(sig_id)
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.ground_disc:
-        signals_ground_disc.sig_passed_button_event(sig_id)
-    return()
-
-def trigger_signal_approach_event (sig_id:int):
-
-    global logging
-    # Validate the signal exists
-    if not signals_common.sig_exists(sig_id):
-        logging.error ("Signal "+str(sig_id)+": trigger_signal_approach_event - Signal does not exist")
-    # call the signal type-specific functions to update the signal
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.colour_light:
-        signals_colour_lights.approach_release_button_event(sig_id)
-    elif signals_common.signals[str(sig_id)]["sigtype"] == signals_common.sig_type.semaphore:
-        signals_semaphores.approach_release_button_event(sig_id)
-    else:
-        logging.error ("Signal "+str(sig_id)+": trigger_signal_approach_event - Function not supported by signal type")
+        logging.error ("Signal "+str(sig_id)+": trigger_timed_signal - Function not supported by signal type")
     return()
 
 ##########################################################################################
