@@ -17,44 +17,6 @@ from . import common
 from tkinter import *
 import logging
 
-# -------------------------------------------------------------------------
-# Define a null callback function for internal use
-# -------------------------------------------------------------------------
-
-def null_callback (sig_id:int,callback_type):
-    return (sig_id,callback_type)
-
-# -------------------------------------------------------------------------
-# Callbacks for processing button pushes - Will also make an external 
-# callback if one was specified when the signal was created. If not, 
-# then the null_callback function will be called to "do nothing"
-# -------------------------------------------------------------------------
-
-def signal_button_event (sig_id:int):
-    global logging
-    logging.info("Signal "+str(sig_id)+": Signal Change Button Event ***************************************")
-    toggle_ground_disc_signal(sig_id)
-    signals_common.signals[str(sig_id)]['extcallback'] (sig_id, signals_common.sig_callback_type.sig_switched)
-    return ()
-
-def sig_passed_button_event (sig_id:int):
-    global logging
-    logging.info("Signal "+str(sig_id)+": Signal Passed Event **********************************************")
-    signals_common.pulse_signal_passed_button (sig_id)
-    signals_common.signals[str(sig_id)]['extcallback'] (sig_id, signals_common.sig_callback_type.sig_passed)
-    return ()
-
-# -------------------------------------------------------------------------
-# Function to toggle the state of a signal - Called following a signal
-# button event (see above). Can also be called externally for to toggle
-# the state of the signal - to enable automated route setting functions
-# -------------------------------------------------------------------------
-
-def toggle_ground_disc_signal (sig_id:int):
-    signals_common.toggle_signal(sig_id)
-    update_ground_disc_signal (sig_id)
-    return ()
-
 # -------------------------------------------------------------------------
 # Externally called function to create a Ground Disc Signal (drawing objects
 # + state). By default the Signal is "NOT CLEAR" (i.e. set to DANGER)
@@ -63,7 +25,7 @@ def toggle_ground_disc_signal (sig_id:int):
 # -------------------------------------------------------------------------
 
 def create_ground_disc_signal (canvas, sig_id:int, x:int, y:int,
-                               sig_callback = null_callback,
+                               sig_callback = None,
                                orientation:int = 0,
                                sig_passed_button: bool = False, 
                                shunt_ahead: bool = False):
@@ -93,9 +55,6 @@ def create_ground_disc_signal (canvas, sig_id:int, x:int, y:int,
         # Create all of the signal elements common to all signal types
         signals_common.create_common_signal_elements (canvas, sig_id, x, y,
                                        signal_type = signals_common.sig_type.ground_disc,
-                                       sig_callback = signal_button_event,
-                                       sub_callback = null_callback,
-                                       passed_callback = sig_passed_button_event,
                                        ext_callback = sig_callback,
                                        orientation = orientation,
                                        sig_passed_button = sig_passed_button)
